import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';

class AssignTagModelBloc
    extends Bloc<AssignTagModelEvent, AssignTagModelState> {
  final List<String> allTags;

  AssignTagModelBloc(this.allTags) : super(AssignTagModelInitial()) {
    on<InitializeTagModels>((event, emit) {
      final initialTags = event.initialTags ?? [];

      final existingTagCounts = <String, int>{};
      for (var tag in initialTags) {
        if (tag.product != null) {
          existingTagCounts[tag.product!.uuid] =
              (existingTagCounts[tag.product!.uuid] ?? 0) + 1;
        }
      }

      final tags = <TagModel>[];

      for (var selectedProduct in event.addedProducts) {
        final existingCount = existingTagCounts[selectedProduct.productId] ?? 0;

        if (selectedProduct.count == 0 ||
            selectedProduct.count <= existingCount) {
          tags.addAll(initialTags
              .where((tag) => tag.product?.uuid == selectedProduct.productId));
          continue;
        }

        final missingCount = selectedProduct.count - existingCount;

        tags.addAll(initialTags
            .where((tag) => tag.product?.uuid == selectedProduct.productId));

        if (missingCount > 0) {
          tags.addAll(List.generate(
            missingCount,
            (index) => TagModel(
              tag: '',
              product: selectedProduct.product,
              location: 'Main Space',
            ),
          ));
        }
      }

      final updatedTags = _calculateAvailableTags(allTags, tags);

      emit(AssignTagModelLoaded(
          tags: tags,
          updatedTags: updatedTags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: ''));
    });

    on<UpdateTag>((event, emit) {
      final currentState = state;
      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags);
        tags[event.index] = tags[event.index].copyWith(tag: event.tag);
        final updatedTags = _calculateAvailableTags(allTags, tags);

        emit(AssignTagModelLoaded(
          tags: tags,
          updatedTags: updatedTags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<UpdateLocation>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags);

        // Use copyWith for immutability
        tags[event.index] =
            tags[event.index].copyWith(location: event.location);

        final updatedTags = _calculateAvailableTags(allTags, tags);

        emit(AssignTagModelLoaded(
          tags: tags,
          updatedTags: updatedTags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<ValidateTagModels>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags);

        emit(AssignTagModelLoaded(
          tags: tags,
          updatedTags: _calculateAvailableTags(allTags, tags),
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      }
    });

    on<DeleteTagModel>((event, emit) {
      final currentState = state;

      if (currentState is AssignTagModelLoaded &&
          currentState.tags.isNotEmpty) {
        final tags = List<TagModel>.from(currentState.tags)
          ..remove(event.tagToDelete);

        final updatedTags = _calculateAvailableTags(allTags, tags);

        emit(AssignTagModelLoaded(
          tags: tags,
          updatedTags: updatedTags,
          isSaveEnabled: _validateTags(tags),
          errorMessage: _getValidationError(tags),
        ));
      } 
    });
  }

  bool _validateTags(List<TagModel> tags) {
    final uniqueTags = tags.map((tag) => tag.tag?.trim() ?? '').toSet();
    final hasEmptyTag = tags.any((tag) => (tag.tag?.trim() ?? '').isEmpty);
    final isValid = uniqueTags.length == tags.length && !hasEmptyTag;
    return isValid;
  }

  String? _getValidationError(List<TagModel> tags) {
    // Check for duplicate tags

    final nonEmptyTags = tags
        .map((tag) => tag.tag?.trim() ?? '')
        .where((tag) => tag.isNotEmpty)
        .toList();

    final duplicateTags = nonEmptyTags
        .fold<Map<String, int>>({}, (map, tag) {
          map[tag] = (map[tag] ?? 0) + 1;
          return map;
        })
        .entries
        .where((entry) => entry.value > 1)
        .map((entry) => entry.key)
        .toList();

    if (duplicateTags.isNotEmpty) {
      return 'Duplicate tags found: ${duplicateTags.join(', ')}';
    }

    return null;
  }

  List<String> _calculateAvailableTags(
      List<String> allTags, List<TagModel> tags) {
    final selectedTags = tags
        .where((tag) => (tag.tag?.trim().isNotEmpty ?? false))
        .map((tag) => tag.tag!.trim())
        .toSet();

    final availableTags =
        allTags.where((tag) => !selectedTags.contains(tag.trim())).toList();
    return availableTags;
  }
}
